﻿using EPiServer.Commerce.Catalog.ContentTypes;
using EPiServer.Commerce.Order;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Workflow.Activities.PurchaseOrderActivities;
using Mediachase.Commerce.WorkflowCompatibility;
using System.Collections.Generic;
using System.Linq;

namespace Mediachase.Commerce.Workflow.Activities.Cart
{
    public class GetFulfillmentWarehouseActivity : HandoffActivityBase
    {
        private static Injected<IFulfillmentWarehouseProcessor> _fulfillmentWarehouseProcessor;

        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            // Validate the properties at runtime
            ValidateRuntime();
            GetFulfillmentWarehouse();
            return ActivityExecutionStatus.Closed;
        }

        private void GetFulfillmentWarehouse()
        {
            var order = (IOrderGroup)OrderGroup;
            var validationIssues = new Dictionary<ILineItem, IList<ValidationIssue>>();

            foreach (var shipment in order.Forms.SelectMany(f => f.Shipments))
            {
                var deletedLineItems = new List<ILineItem>();
                var warehouse = _fulfillmentWarehouseProcessor.Service.GetFulfillmentWarehouse(shipment);
                foreach (var lineItem in shipment.LineItems)
                {
                    if (warehouse != null)
                    {
#pragma warning disable 618
                        ((ILineItemInventory)lineItem).InventoryStatus = (int)GetInventoryStatus(lineItem.Code, warehouse.Code);
#pragma warning restore 618
                    }
                    else
                    {
                        var variation = lineItem.GetEntryContent(ReferenceConverter, ContentLoader);
                        if (variation is VariationContent)
                        {
                            deletedLineItems.Add(lineItem);

                            validationIssues.Add(lineItem, new List<ValidationIssue>() { ValidationIssue.RemovedDueToInactiveWarehouse });
                        }
                    }
                }

                foreach (var lineItem in deletedLineItems)
                {
                    shipment.LineItems.Remove(lineItem);
                }

                if (warehouse != null)
                {
                    shipment.WarehouseCode = warehouse.Code;
                }
            }

            if (validationIssues.Count > 0)
            {
                AddWarningMessages(validationIssues);
            }
        }
    }
}
