using EPiServer.Commerce.Order;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.WorkflowCompatibility;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Mediachase.Commerce.Workflow.Activities
{
    public class CheckInventoryActivity : OrderGroupActivityBase
    {
        /// <summary>
        /// Called by the workflow runtime to execute an activity.
        /// </summary>
        /// <param name="executionContext">The <see cref="T:Mediachase.Commerce.WorkflowCompatibility.ActivityExecutionContext"/> to associate with this <see cref="T:Mediachase.Commerce.WorkflowCompatibility.Activity"/> and execution.</param>
        /// <returns>
        /// The <see cref="T:Mediachase.Commerce.WorkflowCompatibility.ActivityExecutionStatus"/> of the run task, which determines whether the activity remains in the executing state, or transitions to the closed state.
        /// </returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            // Validate the properties at runtime
            ValidateRuntime();

            ValidateItems();

            // Retun the closed status indicating that this activity is complete.
            return ActivityExecutionStatus.Closed;
        }

        /// <summary>
        /// Validate inventory in the order group.
        /// </summary>
        /// <remarks>We don't need to validate quantity in the wishlist.</remarks>
        private void ValidateItems()
        {
            if (string.Equals(OrderGroup.Name, Mediachase.Commerce.Orders.Cart.WishListName, StringComparison.OrdinalIgnoreCase))
            {
                return;
            }

            var validationIssues = new Dictionary<ILineItem, IList<ValidationIssue>>();
            OrderGroup.UpdateInventoryOrRemoveLineItems((item, issue) => AddValidationIssues(validationIssues, item, issue));
            AddWarningMessages(validationIssues);

            // Update line item quantity in order form. This logic to support legacy line items
            var allLineItemInShipment = OrderGroup.GetAllLineItems();
            foreach (var lineItem in OrderGroup.OrderForms.SelectMany(x => x.LineItems))
            {
                var lineItemsInShipment = allLineItemInShipment.Where(x => x.LineItemId == lineItem.LineItemId);
                if (lineItemsInShipment.Any())
                {
                    lineItem.Quantity = lineItemsInShipment.Sum(l => l.Quantity);
                }
            }
        }
    }
}