using EPiServer.Commerce.Catalog.Linking;
using EPiServer.Commerce.Order;
using EPiServer.Globalization;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Catalog;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.WorkflowCompatibility;

namespace Mediachase.Commerce.Workflow.Activities.OrderGroupActivities
{
    public class UpdateTotalsActivity : OrderGroupActivityBase
    {
        private Injected<IOrderGroupCalculator> OrderGroupCalculator = default;
        private Injected<ICatalogSystem> CatalogSystem = default;
        private Injected<IRelationRepository> RelationRepository = default;
        private Injected<LanguageResolver> LanguageResolver = default;

        /// <summary>
        /// Gets the totals, and updates the order group with the calculated values.
        /// </summary>
        /// <param name="executionContext"></param>
        /// <returns><see cref="ActivityExecutionStatus.Closed"/></returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            OrderGroup.CalculateTotals(OrderGroupCalculator.Service, ReferenceConverter, ContentLoader, CatalogSystem.Service, RelationRepository.Service, LanguageResolver.Service);

            foreach (OrderForm orderForm in OrderGroup.OrderForms)
            {
                UpdateOrderFormPaymentTotals(orderForm);
            }

            // Retun the closed status indicating that this activity is complete.
            return ActivityExecutionStatus.Closed;
        }

        /// <summary>
        /// Updates the payment totals for an order form.
        /// </summary>
        /// <param name="orderForm">The order form</param>
        protected virtual void UpdateOrderFormPaymentTotals(OrderForm orderForm) => orderForm.UpdatePaymentTotals();
    }
}