﻿using EPiServer.Commerce.Order;
using EPiServer.Commerce.Order.Internal;
using Mediachase.Commerce.WorkflowCompatibility;
using System;

namespace Mediachase.Commerce.Workflow.Activities.PurchaseOrderActivities
{
    public class CalculatePurchaseOrderStatusActivity : PurchaseOrderBaseActivity
    {
        /// <summary>
        /// Executes the activity.
        /// </summary>
        /// <param name="executionContext">The <see cref="T:Mediachase.Commerce.WorkflowCompatibility.ActivityExecutionContext"/> to associate with this <see cref="T:Mediachase.Commerce.WorkflowCompatibility.Activity"/> and execution.</param>
        /// <returns>
        /// The <see cref="T:Mediachase.Commerce.WorkflowCompatibility.ActivityExecutionStatus"/> of the run task, which determines whether the activity remains in the executing state, or transitions to the closed state.
        /// </returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            if (OrderGroup == null)
            {
                throw new NullReferenceException(nameof(OrderGroup));
            }

            if (OrderGroup is IPurchaseOrder purchaseOrder)
            {
                purchaseOrder.UpdateShipmentsStatus();
            }
            else if (OrderGroup is IPaymentPlan paymentPlan)
            {
                paymentPlan.UpdateShipmentsStatus();
            }

            ChangeOrderStatus();

            // Retun the closed status indicating that this activity is complete.
            return ActivityExecutionStatus.Closed;
        }
    }
}