﻿using EPiServer.Commerce.Order;
using EPiServer.Commerce.Order.Internal;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Orders.Managers;
using System;

namespace Mediachase.Commerce.Workflow.Activities.PurchaseOrderActivities
{
    public abstract class PurchaseOrderBaseActivity : OrderGroupActivityBase
    {
        /// <summary>
        /// Occurs when [changing order status].
        /// </summary>
        public static string ChangingOrderStatusEvent = "ChangingOrderStatus";

        /// <summary>
        /// Occurs when [changing order status].
        /// </summary>
        public static string ChangedOrderStatusEvent = "ChangedOrderStatus";

        protected OrderStatus OrderStatus => OrderStatusManager.GetOrderGroupStatus((IOrderGroup)OrderGroup);

        protected void ChangeOrderStatus(OrderStatus newStatus)
        {
            RaiseEvent(ChangingOrderStatusEvent, EventArgs.Empty);

            OrderGroup.Status = newStatus.ToString();

            RaiseEvent(ChangedOrderStatusEvent, EventArgs.Empty);
        }

        [Obsolete("This method is no longer used. Will remain at least until September 2019.")]
        protected static OrderShipmentStatus GetShipmentStatus(Shipment shipment)
        {
            OrderShipmentStatus retVal = OrderShipmentStatus.InventoryAssigned;
            if (!string.IsNullOrEmpty(shipment.Status))
            {
                retVal = (OrderShipmentStatus)Enum.Parse(typeof(OrderShipmentStatus), shipment.Status);
            }
            return retVal;
        }

        protected void ChangeOrderStatus()
        {
            RaiseEvent(ChangingOrderStatusEvent, EventArgs.Empty);

            if (OrderGroup is IPurchaseOrder purchaseOrder)
            {
                purchaseOrder.UpdateOrderStatus();
            }
            else if (OrderGroup is IPaymentPlan paymentPlan)
            {
                paymentPlan.UpdateOrderStatus();
            }

            RaiseEvent(ChangedOrderStatusEvent, EventArgs.Empty);
        }
    }
}
