﻿using EPiServer.Commerce.Order;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Markets;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.WorkflowCompatibility;
using System.Linq;

namespace Mediachase.Commerce.Workflow.Activities.ReturnForm
{
    public class CalculateReturnFormTotalsActivity : ReturnFormBaseActivity
    {
        private Injected<IReturnOrderFormCalculator> ReturnOrderFormCalculator = default;
        private Injected<IMarketService> MarketService = default;

        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            // Calculate order totals
            CalculateTotalsReturnOrderForm();

            // Return the closed status indicating that this activity is complete.
            return ActivityExecutionStatus.Closed;
        }

        /// <summary>
        /// Calculates the totals for return order form.
        /// </summary>
        private void CalculateTotalsReturnOrderForm()
        {
            var order = ReturnOrderForm.Parent as IPurchaseOrder;
            var market = MarketService.Service.GetMarket(order.MarketId);
            var currency = order.Currency;
            var returnTotals = ReturnOrderFormCalculator.Service.GetReturnOrderFormTotals(ReturnOrderForm, market, currency);

            ReturnOrderForm.Total = returnTotals.Total;
            ReturnOrderForm.SubTotal = returnTotals.SubTotal;
            ReturnOrderForm.TaxTotal = returnTotals.TaxTotal;
            ReturnOrderForm.DiscountAmount = returnTotals.DiscountTotal;
        }
    }
}