﻿using EPiServer.Commerce.Order;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Markets;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Workflow.Activities.ReturnForm;
using Mediachase.Commerce.WorkflowCompatibility;
using System;

namespace Mediachase.Commerce.Workflow.Activities.ReturnFormActivities
{
    public class CreateExchangePaymentActivity : ReturnFormBaseActivity
    {
        private Injected<IOrderRepository> OrderRepository;
        private Injected<IPurchaseOrderProcessor> OrderProcessor;
        private Injected<IMarketService> MarketService;
        private Injected<IPurchaseOrderFactory> PurchaseOrderFactory;

        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            CreateExchangePayments();

            // Return the closed status indicating that this activity is complete.
            return ActivityExecutionStatus.Closed;
        }

        public void CreateExchangePayments()
        {
            var exchangeOrder = ReturnOrderForm.ExchangeOrderGroupId.HasValue ? OrderRepository.Service.Load<IPurchaseOrder>(ReturnOrderForm.ExchangeOrderGroupId.Value) : null;
            if (exchangeOrder != null)
            {
                var origPurchaseOrder = ReturnOrderForm.Parent as IPurchaseOrder;
                var origOrderForm = origPurchaseOrder.GetFirstForm();
                var exchangeOrderForm = exchangeOrder.GetFirstForm();
                // Exchange payments.
                // Credit exchange payment to original order.
                var paymentTotal = Math.Min(ReturnOrderForm.GetTotal(MarketService.Service.GetMarket(origPurchaseOrder.MarketId), origPurchaseOrder.Currency).Amount,
                    exchangeOrder.GetTotal().Amount);
                var creditExchangePayment = CreateExchangePayment(TransactionType.Credit, paymentTotal);
                origOrderForm.Payments.Add(creditExchangePayment);

                // Debit exchange payment to exchange order.
                var debitExchangePayment = CreateExchangePayment(TransactionType.Capture, paymentTotal);
                exchangeOrderForm.Payments.Add(debitExchangePayment);

                OrderProcessor.Service.ProcessOrder(exchangeOrder);
                OrderRepository.Service.Save(exchangeOrder);
            }
        }

        private IPayment CreateExchangePayment(TransactionType type, decimal amount)
        {
            var payment = PurchaseOrderFactory.Service.CreateExchangePayment();
            payment.Amount = amount;
            payment.TransactionType = type.ToString();
            payment.Status = PaymentStatus.Processed.ToString();

            return payment;
        }
    }
}