﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Mediachase.Search;
using System.Threading;

namespace Mediachase.Commerce.Website.Search
{
    public class SelectedFilter
    {
        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name { get; set; }


        /// <summary>
        /// Gets or sets the remove URL.
        /// </summary>
        /// <value>The remove URL.</value>
        public string RemoveUrl { get; set; }

        /// <summary>
        /// Gets or sets the name of the value.
        /// </summary>
        /// <value>The name of the value.</value>
        public string ValueName { get; set; }

        /// <summary>
        /// Gets or sets the filter.
        /// </summary>
        /// <value>The filter.</value>
        public SearchFilter Filter { get; set; }

        /// <summary>
        /// Gets or sets the simple value.
        /// </summary>
        /// <value>The simple value.</value>
        public SimpleValue SimpleValue { get; set; }

        /// <summary>
        /// Gets or sets the range value.
        /// </summary>
        /// <value>The range value.</value>
        public RangeValue RangeValue { get; set; }

        /// <summary>
        /// Gets or sets the price range value.
        /// </summary>
        /// <value>The price range value.</value>
        public PriceRangeValue PriceRangeValue { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectedFilter"/> class.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="value">The value.</param>
        public SelectedFilter(SearchFilter filter, SimpleValue value)
        {
            Filter = filter;
            SimpleValue = value;
            Name = SearchCommon.GetDescription(Thread.CurrentThread.CurrentCulture.Name, filter.Descriptions);
            ValueName = SearchCommon.GetDescription(Thread.CurrentThread.CurrentCulture.Name, value.Descriptions);
            RemoveUrl = SearchFilterHelper.GetQueryStringNavigateUrl(filter.field, true);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectedFilter"/> class.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="value">The value.</param>
        public SelectedFilter(SearchFilter filter, RangeValue value)
        {
            Filter = filter;
            RangeValue = value;
            Name = SearchCommon.GetDescription(Thread.CurrentThread.CurrentCulture.Name, filter.Descriptions);
            ValueName = SearchCommon.GetDescription(Thread.CurrentThread.CurrentCulture.Name, value.Descriptions);
            RemoveUrl = SearchFilterHelper.GetQueryStringNavigateUrl(filter.field, true);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectedFilter"/> class.
        /// </summary>
        /// <param name="filter">The filter.</param>
        /// <param name="value">The value.</param>
        public SelectedFilter(SearchFilter filter, PriceRangeValue value)
        {
            Filter = filter;
            PriceRangeValue = value;
            Name = SearchCommon.GetDescription(Thread.CurrentThread.CurrentCulture.Name, filter.Descriptions);
            ValueName = SearchCommon.GetDescription(Thread.CurrentThread.CurrentCulture.Name, value.Descriptions);
            RemoveUrl = SearchFilterHelper.GetQueryStringNavigateUrl(filter.field, true);
        }
    }

}
